# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-28 08:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix', '0016_auto_20170419_1533'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='action_flag',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Add'), (2, 'Change'), (3, 'Delete')], verbose_name='Action'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

try:
    from subprocess import getstatusoutput
    pythoncmd="python3"
except:
    from commands import getstatusoutput
    pythoncmd="python"

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from codenerix.lib.debugger import Debugger

class Command(BaseCommand, Debugger):

    # Show this when the user types help
    help = "Used to build all locales details"
    
    def add_arguments(self, parser):
        # Set witch mode to use
        parser.add_argument('--mode',
            dest='mode',
            default="suexec",
            help="Mode used in the environment 'suexec', 'apache' or 'wwwdata'") 
        
        # Named (optional) arguments
        parser.add_argument('--noauto',
            action='store_true',
            dest='noauto',
            default=False,
            help='Tells the command not to find automatic solution for problems')
        
        # Named (optional) arguments
        parser.add_argument('--noguess',
            action='store_true',
            dest='noguess',
            default=False,
            help='Disable guessing user environment')
        
        # Named (optional) arguments
        parser.add_argument('--clean',
            action='store_true',
            dest='clean',
            default=False,
            help='Do a full clean by deleting everything before building translations')
    
    def handle(self, *args, **options):
        # Check arguments
        mode=options['mode']
        if mode not in ['wwwdata','apache','suexec']:
            self.print_help('','')
            raise CommandError("Allowed modes are suexec, apache or wwwdata ('apache' and 'wwwdata' option will force permissions to apache or www-data user, suexec won't touch permissions)")
        
        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()
        
        # Get environment
        appname=settings.ROOT_URLCONF.split(".")[0]
        basedir=settings.BASE_DIR
        appdir = os.path.abspath("{}/{}".format(basedir,appname))
        noauto=options['noauto']
        
        # Check user selection
        if not options['noguess']:
            cmd = "find {} -name 'locale' -exec ls -lR {{}} \; | grep www-data".format(appdir)
            status, output = getstatusoutput(cmd)
            if status:
                guess="suexec"
            else:
                guess="wwwdata"
            if guess!=mode:
                self.print_help('','')
                raise CommandError("You have selected mode '{}' but I believe you are using '{}', if sure use --noguess".format(mode,guess))
        
        # Show header
        self.debug("Creating locales for {}".format(appname),color='blue')
        if noauto:
            self.debug("Autoconfig mode is OFF",color='yellow')
        
        # Get list of apps
        apps=[]
        length=len(appname)
        for app in settings.INSTALLED_APPS:
            if app[0:length]==appname:
                apps.append(app[length+1:])
        
        # Check for locale folders
        ERROR=False
        for app in ['']+apps:
            testpath = os.path.abspath("{}/{}/locale".format(appdir,app).replace("//","/"))
            if not os.path.exists(testpath):
                if noauto:
                    ERROR=True
                    self.debug("'locale' folder missing at {}/".format(testpath),color='yellow')
                else:
                    self.debug("'locale' folder missing, creating {}/".format(testpath),color='purple')
                    os.mkdir(testpath)
        
        if ERROR:
            raise CommandError("Some error has happened, can not keep going! (avoid using --noauto to let CODENERIX find a solution)")
        
        # Check execution mode
        sudo=''
        if mode=='apache' or mode=='wwwdata':
            status, output = getstatusoutput("whoami")
            if status:
                # Error in command
                self.error("Error while executing 'whoami' command")
                raise CommandError(output)
            elif output=='www-data':
                # Detected we are already www-data user, so keep going
                self.debug("Detected we are 'www-data' user",color='purple')
            else:
                # No permissions try to become root using sudo
                status, output = getstatusoutput("sudo whoami")
                if status:
                    # Error in command
                    self.error("Error while executing 'sudo whoami' command")
                    raise CommandError(output)
                elif output=='root':
                    # Detected sudo is working and we can become root
                    self.debug("Detected we can become 'root' user",color='purple')
                    sudo='sudo '
                else:
                    # No permissions
                    raise CommandError("You requested 'apache' or 'wwwdata' execution mode but you are not www-data and we can not become root")
        
        # Delete locale folders if requested
        if options['clean']:
            
            # Ask the user if would really like to remove all locale folders
            key = ''
            while key not in ['n', 'y']:
                self.debug("All 'locale' folders are going to be removed, are you sure? (y|n) ", tail=False, color="red")
                try:
                    key = raw_input().lower()
                except NameError:
                    key = input().lower()
                self.debug("", header=False)
            
            # Remove all locale folders
            if key=='y':
                self.debug("Removing locale folders...",color="red")
                for app in ['']+apps:
                    testpath = os.path.abspath("{}/{}/locale".format(appdir,app).replace("//","/"))
                    if os.path.exists(testpath):
                        self.debug("    > Removing {}".format(testpath),color="red")
                        cmd = "{}rm -R {}/*".format(sudo,testpath)
                        status, output = getstatusoutput(cmd)
                        if status: raise CommandError(output)
            else:
                raise CommandError("You requested to clean all 'locale' folders but you answered NO to the previous question, can not keep going!")
        
        
        # Ready to go
        for (code,name) in settings.LANGUAGES:
            self.debug("Processing translations for {}...".format(name), color='cyan')
            cmd="{}{}/manage.py makemessages -v0 --symlinks --ignore env -l {}".format(sudo,basedir,code) # --exclude is not working in some computers
            status, output = getstatusoutput(cmd)
            if status: raise CommandError(output)
            
            cmd="{}{}/manage.py makemessages -v0 --symlinks --ignore env -d djangojs -l {}".format(sudo,basedir,code) # --exclude is not working in some computers
            status, output = getstatusoutput(cmd)
            if status: raise CommandError(output)
            
        
        # Set permissions for locale folders
        if sudo:
            self.debug("Setting permissions...",color='cyan')
            if mode=='apache':
                user='apache'
            elif mode=='wwwdata':
                user='www-data'
            else:
                raise CommandError("Wrong mode for sudo '{}'".format(mode))
            for app in ['']+apps:
                testpath = os.path.abspath("{}/{}/locale".format(appdir,app).replace("//","/"))
                cmd="sudo chown {}.{} {}/ -R".format(user,user,testpath)
                status, output = getstatusoutput(cmd)
                if status: raise CommandError(output)




# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# System
import os
from datetime import datetime
import random
from dateutil.tz import tzutc
import zipfile
import io
import json
from unidecode import unidecode

# Django
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from django.template import TemplateDoesNotExist
from django.template.loader import get_template as django_get_template
from django.shortcuts import render
from django.conf import settings
from django.views.generic.base import View
from django.core.cache import cache
from django.utils import dateparse


def epochdate(timestamp):
    '''
    Convet an epoch date to a tuple in format ("yyyy-mm-dd","hh:mm:ss")
    Example: "1023456427" -> ("2002-06-07","15:27:07")

    Parameters:
    - `timestamp`: date in epoch format
    '''

    dt = datetime.fromtimestamp(float(timestamp)).timetuple()
    fecha = "{0:d}-{1:02d}-{2:02d}".format(dt.tm_year, dt.tm_mon, dt.tm_mday)
    hora = "{0:02d}:{1:02d}:{2:02d}".format(dt.tm_hour, dt.tm_min, dt.tm_sec)
    return (fecha, hora)


def date2string(dtime, formt, default):
    try:
        list_type = [str, unicode, ]
    except NameError:
        list_type = [str, ]

    if dtime:
        if ('year' in dir(dtime)) and (dtime.year < 1900):
            return default
        elif type(dtime) in list_type:
            return dtime
        else:
            return dtime.strftime(formt)
    else:
        return default


def daterange_filter(value, variable):
    start = dateparse.parse_datetime(value['startDate'])
    end = dateparse.parse_datetime(value['endDate'])
    result = {'{0}__gte'.format(variable): start, '{0}__lte'.format(variable): end}
    return result


def timezone_serialize(d, prettify=False):
    if d is None:
        result = None
    else:
        if d.tzinfo:
            d = d.astimezone(tzutc()).replace(tzinfo=None)
        if prettify:
            result = d.strftime(settings.DATETIME_INPUT_FORMATS[0])
        else:
            result = d.isoformat() + "Z"
    return result


def timezone_deserialize(string):
    if string is None:
        result = None
    else:
        result = dateparse.parse_datetime(string)
    return result


def zeropad(number, width):
    return '{number:0{width}d}'.format(width=width, number=number)


def monthname(value):
    if value == 1:
        return _("January")
    elif value == 2:
        return _("February")
    elif value == 3:
        return _("March")
    elif value == 4:
        return _("April")
    elif value == 5:
        return _("May")
    elif value == 6:
        return _("June")
    elif value == 7:
        return _("July")
    elif value == 8:
        return _("August")
    elif value == 9:
        return _("September")
    elif value == 10:
        return _("Octuber")
    elif value == 11:
        return _("November")
    elif value == 12:
        return _("December")
    else:
        return None


# Name unify
def nameunify(name, url=False):
    # Make unicode
    name = smart_text(name)

    # Get it on lower
    namelow = unidecode(name).lower().strip(' \t\n\r')

    # Define valid characters
    allowed_characters = 'abcdefghijklmnopqrstuvwxyz0123456789_-'
    if url:
        namelow = namelow.replace(' ', "-")
    else:
        namelow = namelow.replace("-", "").replace(' ', '_')

    # Replace all unknown characters
    result = ''
    for c in namelow:
        if c in allowed_characters:
            result += c

    if result[-1] in ("-", "_"):
        result = nameunify(result[0:-1], url)
    # Return the cleaned result
    return result


def get_profile(user):
    # Check if it has admin rights admin
    try:
        is_admin = bool(user.is_superuser or user.groups.get(name='Admins'))
    except Exception:
        is_admin = False

    if is_admin:
        # Administrator use root
        profile = "admin"
    else:
        profile = None

    # Return profile
    return profile


def get_profiled_paths(path, user, lang, extension):
    paths = []
    if user:
        # Check if it has admin rights admin
        profile = get_profile(user)

        # Define the base_path to use
        paths.append("{0}.{1}".format(user.username, lang))
        paths.append(user.username)
        if profile:
            paths.append("{0}.{1}".format(profile, lang))
            paths.append(profile)
        if profile != 'admin':
            paths.append("user.{0}".format(lang))
            paths.append('user')

    # Add an empty path
    paths.append(lang)
    paths.append("")

    # Split path name
    pathsp = path.split(".")
    if pathsp[-1] == extension:
        basepath = ".".join(pathsp[:-1])
    else:
        basepath = path

    # Return paths and basepath
    return (paths, basepath)


def get_template(template, user, lang, extension='html', raise_error=True):
    # Get profiled paths
    (templates, templatepath) = get_profiled_paths(template, user, lang, extension)

    # Check templates
    test = []
    found = None
    for temp in templates:
        if len(temp):
            addon = ".%s" % (temp)
        else:
            addon = ""
        target = "%s%s.%s" % (templatepath, addon, extension)
        test.append(target)
        try:
            django_get_template(target)
            found = target
            break
        except TemplateDoesNotExist:
            pass

    # Return target template
    if found:
        return found
    else:
        if raise_error:
            raise IOError("I couldn't find a valid template, I have tried: {}".format(test))
        else:
            return test


def get_static(desired, user, lang, default, extension='html', relative=False):
    # Get profiled paths
    (paths, basepath) = get_profiled_paths(desired, user, lang, extension)

    # Check paths
    found = None
    for temp in paths:
        if len(temp):
            addon = ".%s" % (temp)
        else:
            addon = ""
        target = "%s%s.%s" % (basepath, addon, extension)
        if hasattr(settings, "STATIC_ROOT") and settings.STATIC_ROOT and os.path.exists(os.path.join(settings.STATIC_ROOT, target)):
            if relative:
                found = os.path.join(settings.STATIC_URL, target)
            else:
                found = target
            break

    # Return target template
    if found:
        # Return the one we found
        return found
    else:
        # We didn't find any, return default one
        return default


def direct_to_template(request, template):
    template = get_template(template, request.user)
    return render(request, template)


def model_inspect(obj):
        '''
        Analize itself looking for special information, right now it returns:
        - Application name
        - Model name
        '''
        # Prepare the information object
        info = {}
        if hasattr(obj, '_meta'):
            info['verbose_name'] = getattr(obj._meta, 'verbose_name', None)
        else:
            info['verbose_name'] = None

        # Get info from the object
        if hasattr(obj, 'model') and obj.model:
            model = obj.model
        else:
            model = obj.__class__

        namesp = str(model)
        namesp = namesp.replace("<class ", "").replace(">", "").replace("'", "").split(".")

        # Remember information
        info['appname'] = namesp[-3]
        info['modelname'] = namesp[-1]
        info['model'] = model

        # Return the info
        return info


def upload_path(instance, filename):
    '''
    This method is created to return the path to upload files. This path must be
    different from any other to avoid problems.
    '''
    path_separator = "/"
    date_separator = "-"
    ext_separator = "."
    empty_string = ""
    # get the model name
    model_name = model_inspect(instance)['modelname']

    # get the string date
    date = datetime.now().strftime("%Y-%m-%d").split(date_separator)
    curr_day = date[2]
    curr_month = date[1]
    curr_year = date[0]

    split_filename = filename.split(ext_separator)
    filename = empty_string.join(split_filename[:-1])
    file_ext = split_filename[-1]

    new_filename = empty_string.join([filename, str(random.random()).split(ext_separator)[1]])
    new_filename = ext_separator.join([new_filename, file_ext])
    string_path = path_separator.join([model_name, curr_year, curr_month, curr_day, new_filename])
    # the path is built using the current date and the modelname
    return string_path


def get_class(func):
    if not getattr(func, '__closure__', None):
        return

    for closure in func.__closure__:
        contents = closure.cell_contents

        if not contents:
            continue

        if getattr(contents, '__bases__', None) and issubclass(contents, View):
            return contents

        result = get_class(contents)
        if result:
            return result


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def clean_memcache_item(key, item):
    # clean item from key
    result = cache.get(key)
    if result is not None and item in result:
        result.pop(item)
        cache.set(key, result)


class CodenerixEncoder(object):

    codenerix_numeric_dic = {
        # Basic dicts
        'num': '0123456789',
        'alpha': 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
        'alphanc': 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',
        'alphanum': '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ',
        'alphanumnc': '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',
        # Original dicts
        'hex36': 'ENWR6MV71JOQHADUGFCYZ25X3B4L0KPTSI98',
        'hex16': '54BEF80D1C96A732',
        'hexz17': 'Z0123456789ABCDEF',
    }

    def list_encoders(self):
        return self.codenerix_numeric_dic.keys()

    def numeric_encode(self, number, dic='hex36', length=None, cfill=None):

        # Get predefined dictionary
        if dic in self.codenerix_numeric_dic:
            dic = self.codenerix_numeric_dic[dic]

        # Integrity check to dic
        nr = ""
        for c in dic:
            if c not in nr:
                nr += c
            else:
                raise IOError(_("ERROR: dic has repeated elements"))

        # If no cfill
        if cfill is None:
            cfill = dic[0]

        # Find lenght
        ldic = len(dic)

        # Initialize
        string = ""
        div = ldic + 1

        # Process number
        while div >= ldic:
            div = int(number / ldic)
            mod = number % ldic
            string += dic[mod]
            number = div

        # If something left behind
        if div:
            string += dic[div]

        # Fill the string if requested
        if length:
            string += cfill * (length - len(string))

        # Return the reverse string
        return string[::-1]

    def numeric_decode(self, string, dic='hex36'):

        # Get predefined dictionary
        if dic in self.codenerix_numeric_dic:
            dic = self.codenerix_numeric_dic[dic]

        # For each character in the string
        first = True
        number = 0
        for c in string:

            # Not the first loop
            if not first:
                number *= len(dic)
            else:
                first = False

            # Attach the character
            number += dic.index(c)

        # Return the final resulting number
        return number


class InMemoryZip(object):
    '''
    # Compress
    imz = InMemoryZip()
    imz.append("info.dat", data).append("test.txt","asdfasfdsaf")
    datazip = imz.read()

    # Get FILE pointer and read() from it
    FILE = imz.get_fp()
    FILE.read()

    # Uncompress
    imz = InMemoryZip(datazip)$
    info_unzipped = imz.get("info.dat")
    '''

    def __init__(self, data=None):
        # Create the in-memory file-like object
        self.in_memory_zip = io.BytesIO()
        if data:
            self.in_memory_zip.write(data)
            self.in_memory_zip.seek(0)

    def get_fp(self):
        self.in_memory_zip.seek(0)
        return self.in_memory_zip

    def append(self, filename_in_zip, file_contents):
        '''
        Appends a file with name filename_in_zip and contents of
        file_contents to the in-memory zip.
        '''
        # Set the file pointer to the end of the file
        self.in_memory_zip.seek(-1, io.SEEK_END)

        # Get a handle to the in-memory zip in append mode
        zf = zipfile.ZipFile(self.in_memory_zip, "a", zipfile.ZIP_DEFLATED, False)

        # Write the file to the in-memory zip
        zf.writestr(filename_in_zip, file_contents)

        # Mark the files as having been created on Windows so that
        # Unix permissions are not inferred as 0000
        for zfile in zf.filelist:
            zfile.create_system = 0

        # Close the ZipFile
        zf.close()

        # Rewind the file
        self.in_memory_zip.seek(0)

        return self

    def get(self, filename):
        self.in_memory_zip.seek(0)
        zf = zipfile.ZipFile(self.in_memory_zip, "r", zipfile.ZIP_DEFLATED, False)
        fp = zf.open(filename)
        data = fp.read()
        # Close the ZipFile
        zf.close()
        # Return the result
        return data

    def size(self):
        size = self.in_memory_zip.seek(-1, io.SEEK_END)
        self.in_memory_zip.seek(0)
        return size

    def read(self):
        '''Returns a string with the contents of the in-memory zip.'''
        self.in_memory_zip.seek(0)
        return self.in_memory_zip.read()

    def writetofile(self, filename):
        '''Writes the in-memory zip to a file.'''
        f = open(filename, "w")
        f.write(self.read())
        f.close()


def remove_getdisplay(field_name):
    '''
    for string 'get_FIELD_NAME_display' return 'FIELD_NAME'
    '''
    str_ini = 'get_'
    str_end = '_display'
    if str_ini == field_name[0:len(str_ini)] and str_end == field_name[(-1) * len(str_end):]:
        field_name = field_name[len(str_ini):(-1) * len(str_end)]
    return field_name


def trace_json_error(struct, path=[]):
    found = None
    kind = type(struct)
    if kind is dict:
        for key in struct:
            found = trace_json_error(struct[key], path+[key])
            if found:
                break
    elif kind is list:
        idx = 0
        for element in struct:
            found = trace_json_error(element, path+[str(idx)])
            if found:
                break
            idx += 1
    else:
        try:
            json.dumps(struct)
        except Exception:
            found = path

    # Return result
    return found



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
'''
Colors definition
'''

__version__ = "201109111106"

__all__ = ['colors']


colors={}

colors['simplegrey']   = (0, 30)
colors['simplered']    = (0, 31)
colors['simplegreen']  = (0, 32)
colors['simpleyellow'] = (0, 33)
colors['simpleblue']   = (0, 34)
colors['simplepurple'] = (0, 35)
colors['simplecyan']   = (0, 36)
colors['simplewhite']  = (0, 37)

colors['grey']   = (1, 30)
colors['red']    = (1, 31)
colors['green']  = (1, 32)
colors['yellow'] = (1, 33)
colors['blue']   = (1, 34)
colors['purple'] = (1, 35)
colors['cyan']   = (1, 36)
colors['white']  = (1, 37)

colors['close']  = (1, 0)

def colorize(msg,color=None):
    # Colors
    if color in colors:
        (darkbit,subcolor)=colors[color]
    else:
        (darkbit,subcolor)=(1,0)
    
    # Prepare the message
    result ="\033[%1d;%02dm" % (darkbit,subcolor)
    result+=msg
    result+="\033[%1d;%02dm" % (1,0)
    
    # Return the result
    return result

if __name__ == "__main__":
    
    # Reorder colors
    keys = []
    for key in colors.keys():
        keys.append((colors[key][0],colors[key][1],key))
    keys.sort()
    
    # Show up all colors
    for color in keys:
        # Get the color information
        (simplebit, subcolor) = colors[color[2]]
        # Show it
        print("{0:1d}:{1:02d} - \033[{2:1d};{3:02d}m{4:<14s}\033[1;00m{5:<s}".format(simplebit, subcolor, simplebit, subcolor, color[2], color[2]))




# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from django.apps import apps
from django.core.management.base import BaseCommand
from django.contrib.auth.models import User
from django.contrib.auth.management import create_permissions
from django.contrib.contenttypes.management import update_contenttypes

from codenerix.lib.debugger import Debugger


class Command(BaseCommand, Debugger):

    # Show this when the user types help
    help = "Refresh all permissions from a project"

    def handle(self, *args, **options):

        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()
        self.debug("Settings permissions for:", color='blue')

        # Get list of apps
        self.debug("Getting list of APPs", color='blue')
        apps_config = apps.get_app_configs()
        apps_total = len(apps_config)

        # Create missins permissions
        self.debug("Creating missing permissions", color='blue')
        idx = 1
        for app_config in apps_config:
            self.debug("    -> {}/{} {}".format(idx, apps_total, app_config.label), color='cyan')
            create_permissions(app_config, apps=apps, verbosity=0)
            idx += 1

        # Update contenttypes
        self.debug("Updating Content Types", color='blue')
        idx = 1
        for app_config in apps_config:
            self.debug("    -> {}/{} {}".format(idx, apps_total, app_config.label), color='cyan')
            update_contenttypes(app_config)
            idx += 1

        # Get all users from the system
        person = None
        for user in User.objects.all():
            self.debug("    > {} ".format(user.username), color='cyan', tail=None)
            if hasattr(user, 'person') and user.person:
                self.debug("OK".format(user.username), color='green', header=None)
                user.person.refresh_permissions()
                person = user.person
            else:
                self.debug("NO PERSON".format(user.username), color='red', header=None)

        # Remake groups permissions if we have at least one valid user
        if person:
            self.debug("Refreshing group permissions (it may takes over a minute)... ", color='blue', tail=None)
            person.__class__.group_permissions(person.__class__)
            self.debug("DONE", color='green', header=None)
        else:
            self.debug("Can not refresh group permissions because I didn't find a user with a Person", color='red')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-08 11:38
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix', '0010_corporateimage'),
    ]




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('action_time', models.DateTimeField(auto_now=True, verbose_name='Date')),
                ('object_id', models.TextField(null=True, verbose_name='Object id', blank=True)),
                ('object_repr', models.CharField(max_length=200, verbose_name='Object repr')),
                ('action_flag', models.PositiveSmallIntegerField(verbose_name='Acci\xf3n')),
                ('change_json', models.TextField(verbose_name='Json', blank=True)),
                ('change_txt', models.TextField(verbose_name='Txt', blank=True)),
                ('content_type', models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True)),
                ('user', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'permissions': (('list_log', 'Can list log'), ('detail_log', 'Can view log')),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-07-13 15:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix', '0017_auto_20170428_0850'),
    ]

    operations = [
        migrations.AddField(
            model_name='log',
            name='snapshot_txt',
            field=models.TextField(blank=True, verbose_name=b'Snapshot Txt'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import socket
from datetime import datetime

from django import forms
from django.db import models
from django.conf import settings
from django.forms.utils import ValidationError
from django.utils.translation import ugettext as _
from django.forms.widgets import Textarea
from django.utils.encoding import smart_text

from multi_email_field.forms import MultiEmailField as MultiEmailFormField
from captcha import client

from codenerix.widgets import FileAngularInput, Date2TimeInput, WysiwygAngularInput, GenReCaptchaInput, MultiBlockWysiwygInput, BootstrapWysiwygInput


class FileAngularField(models.FileField):
    description = "File manage throught the Angular service system"

    def formfield(self, **kwargs):
        defaults = {'widget': FileAngularInput}
        defaults.update(kwargs)
        return super(FileAngularField, self).formfield(**defaults)


class ImageAngularField(models.ImageField):
    description = "Image field for Angular JS"

    def formfield(self, **kwargs):
        defaults = {'widget': FileAngularInput}
        defaults.update(kwargs)
        return super(ImageAngularField, self).formfield(**defaults)


class Date2TimeField(models.DateTimeField):
    def formfield(self, **kwargs):
        defaults = {'widget': Date2TimeInput}
        defaults.update(kwargs)
        return super(Date2TimeField, self).formfield(**defaults)

    def clean(self, *args, **kwargs):
        date_min = datetime(2000, 1, 1, 0, 0, 0)

        if args[0]:
            if type(args[0]) is not datetime:
                raise ValidationError(_("Invalid date"))
            elif args[0] and date_min.date() > args[0].date():
                raise ValidationError(_("Date too old"))
            # return args[0]
        return super(Date2TimeField, self).clean(*args, **kwargs)


email_er = "[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*"


class MultiEmailField(MultiEmailFormField):
    message = _("Enter valid email addresses.")
    widget = Textarea(attrs={'ng-pattern': "/^({0})(\\n{0})*$/".format(email_er)})

    def to_python(self, value):
        if value:
            for char in [',', ' ', '\r']:
                value = value.replace(char, "\n").replace("\n\n", "\n")
        return super(MultiEmailField, self).to_python(value)


class WysiwygAngularField(models.TextField):
    description = "A hand of cards (bridge style)"

    def formfield(self, **kwargs):
        defaults = {'widget': WysiwygAngularInput}
        defaults.update(kwargs)
        return super(WysiwygAngularField, self).formfield(**defaults)


class MultiBlockWysiwygField(models.TextField):
    description = "Multi block WYSIWYG"

    def formfield(self, **kwargs):
        defaults = {'widget': MultiBlockWysiwygInput}
        defaults.update(kwargs)
        return super(MultiBlockWysiwygField, self).formfield(**defaults)


class BootstrapWysiwygField(models.TextField):
    description = "Bootstrap WYSIWYG"

    def formfield(self, **kwargs):
        defaults = {'widget': BootstrapWysiwygInput}
        defaults.update(kwargs)
        return super(BootstrapWysiwygField, self).formfield(**defaults)


class GenReCaptchaField(forms.CharField):
    description = "ReCaptcha field throught the Angular service system"
    default_error_messages = {
        'captcha_invalid': _('Incorrect, please try again.'),
        'captcha_error': _('Error verifying input, please try again.'),
    }

    def __init__(self, fieldname=None, public_key=None, private_key=None, use_ssl=None, attrs={}, *args, **kwargs):
        """
        ReCaptchaField can accepts attributes which is a dictionary of
        attributes to be passed to the ReCaptcha widget class. The widget will
        loop over any options added and create the RecaptchaOptions
        JavaScript variables as specified in
        https://code.google.com/apis/recaptcha/docs/customization.html

        On compiled verstion of the library, don't forget to add before the codenerix.js scripts to add:
        <script type="text/javascript" src="https://www.google.com/recaptcha/api.js?onload=vcRecaptchaApiLoaded&render=explicit" async defer></script>$
        """
        public_key = public_key if public_key else \
            settings.RECAPTCHA_PUBLIC_KEY
        self.private_key = private_key if private_key else \
            settings.RECAPTCHA_PRIVATE_KEY
        self.use_ssl = use_ssl if use_ssl is not None else getattr(
            settings, 'RECAPTCHA_USE_SSL', False)

        self.widget = GenReCaptchaInput(fieldname=fieldname, public_key=public_key, use_ssl=self.use_ssl, attrs=attrs)
        self.required = True
        return super(GenReCaptchaField, self).__init__(*args, **kwargs)

    def get_remote_ip(self):
        f = sys._getframe()
        while f:
            if 'request' in f.f_locals:
                request = f.f_locals['request']
                if request:
                    remote_ip = request.META.get('REMOTE_ADDR', '')
                    forwarded_ip = request.META.get('HTTP_X_FORWARDED_FOR', '')
                    ip = remote_ip if not forwarded_ip else forwarded_ip
                    return ip
            f = f.f_back

    def clean(self, values):
        super(GenReCaptchaField, self).clean(values[0])
        recaptcha_challenge_value = smart_text(values[0])
        recaptcha_response_value = smart_text(values[1])

        if os.environ.get('RECAPTCHA_TESTING', None) == 'True' and recaptcha_response_value == 'PASSED':
            return values[0]

        try:
            check_captcha = client.submit(
                recaptcha_challenge_value,
                recaptcha_response_value, private_key=self.private_key,
                remoteip=self.get_remote_ip(), use_ssl=self.use_ssl)

        except socket.error:  # Catch timeouts, etc
            # raise IOError,"ERROR: {}".format(self.error_messages['captcha_error'])
            raise ValidationError(
                self.error_messages['captcha_error']
            )

        if not check_captcha.is_valid:
            # raise IOError,"INVALID: {}".format(self.error_messages['captcha_invalid'])
            raise ValidationError(
                self.error_messages['captcha_invalid']
            )

        return values[0]



__version__ = "1.0.74"

__authors__ = [
    'Juan Miguel Taboada Godoy <juanmi@juanmitaboada.com>',
    'Juan Soler Ruiz <soleronline@gmail.com>',
    'Francisco Torrejon Puente <frantorrejon@gmail.com>',
]

__requirements__ = {
    'all':[
        "pymongo",
        "django-angular==0.8.4",
        "python-dateutil",
        "django-recaptcha>=1.2.1,<1.3",
        "django-rosetta",
        "jsonfield",
        "openpyxl",
        "Pillow",
        "Unidecode",
        "Django>=1.10.6,<1.11",
        "django-multi-email-field",
        "ldap3",
        "django-haystack>=2.6.1",
        "pytz",
        "elasticsearch>=2.0.0,<3.0.0",
    ],
    '2':[
        "html5lib==1.0b8", # Default version 0.99999999 is broken with error 'from html5lib import treebuilders, inputstream' => 'ImportError: cannot import name inputstream' (1.0b10 also fails)
    ],
    '3':[
        "html5lib",
        ],
    }



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext as _

WEIGHT_UNITS = {
        # SI
        'kg': (_('Kilogram'),1.0),              # BASE
        'g': (_('Gram'), 1000.0),
        # US/UK
        'lb': (_('Pound'), 2.2046228),
        'oz': (_('Ounce'), 35.273962),
        'st': (_('Stone'), 0.15747304),
    }

LENGTH_UNITS = {
        # SI
        'm': (_('Metre'), 1.0),                 # BASE
        'cm': (_('Centimetre'), 100.0),
        'km': (_('Kilometre'), 0.001),
        # US/UK
        'mi': (_('Mile'), 0.00062137119),
        'yd': (_('Yard'), 1.0936133), 
        'ft': (_('Feet'), 3.2808399),
        'in': (_('Inch'), 39.3701),
    }

VOLUME_UNITS = {
        # SI
        'm3': (_('Cubic metre'), 1.0),          # BASE
        'dm3': (_('Cubic decimetre'), 1000.0),
        'l': (_('Litre'), 1000.0),
        'ml': (_('Mililitre'), 1000000.0),
        # UK
        'ig': (_('UK Gallon'), 219.969),
        'ip': (_('UK Pint'), 1759.75),
        # US
        'g': (_('US Gallon'), 264.172),
        'p': (_('US Pint'), 2113.38),
        # US/UK
        'ft3': (_('Cubic feet'), 35.314666882523476),
        'yd3': (_('Cubic yard'), 1.3079506252786468),
        }
